package controllers

import (
	"bytes"
	"encoding/base64"
	"encoding/json"
	"io/ioutil"
	"net/http"
	"strings"
	"wechat-conf/models"
	"wechat-conf/models/model"
	"wechat-conf/service/autoreply"
	"wechat-conf/service/wechat"
	"wechat-conf/utils"

	"github.com/astaxie/beego"

	"github.com/kinisky564477/wechat/component"

	"github.com/kinisky564477/wechat/core"
)

// WechatController 用户
type WechatController struct {
	BaseController
	serv       *autoreply.AutoreplyServ
	wechatServ *wechat.WechatServ
}

var nilData = []byte("success")

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *WechatController) Constructor() {
	c.serv = autoreply.NewAutoreplyServ(c.Context)
	c.wechatServ = wechat.NewWechatServ(c.Context)
}

const (
	INFOTYPE_TICKET           = "component_verify_ticket"
	INFOTYPE_AUTHORIZED       = "authorized"
	INFOTYPE_UNAUTHORIZED     = "unauthorized"
	INFOTYPE_UPDATEAUTHORIZED = "updateauthorized"
)

// ComponentPush 第三方平台推送
func (c *WechatController) ComponentPush() {
	utils.LogError("推送开始")
	r := c.Ctx.Request
	defer r.Body.Close()
	con, _ := ioutil.ReadAll(r.Body)
	utils.LogError("推送消息：", string(con))
	// 解析xml
	xp := &core.XMLParse{}
	resMsg, err := xp.Parse(string(con))
	if err != nil {
		utils.LogError("xml解析失败：", err)
		c.ResponseRaw(nilData)
	}
	encrypt := resMsg["Encrypt"]
	conf, err := c.wechatServ.GetComponentInfo()
	if err != nil || conf == nil || conf.Appid == "" {
		utils.LogError("读取微信配置文件失败")
		c.ResponseRaw(nilData)
	}
	EncodingAESKey := conf.Aeskey

	AESKey, err := base64.StdEncoding.DecodeString(EncodingAESKey + "=")
	if err != nil {
		utils.LogError("DecodeString失败：", err)
		c.ResponseRaw(nilData)
	}
	EncryptVal, err := base64.StdEncoding.DecodeString(encrypt)
	if err != nil {
		utils.LogError("密文base64解析", err)
		c.ResponseRaw(nilData)
	}
	msgbyte, err := utils.AesDecrypt(EncryptVal, AESKey)
	if err != nil {
		utils.LogError("解密失败：", err)
		c.ResponseRaw(nilData)
	}
	utils.LogError("解密成功")
	// 解析xml
	msg, err := xp.Parse(string(msgbyte))
	if err != nil {
		utils.LogError("xml解析失败：", err)
		c.ResponseRaw(nilData)
	}
	utils.LogError("xml解析成功：", msg)
	switch msg["InfoType"] {
	case INFOTYPE_TICKET:
		// 更新ticket
		conf.Ticket = msg["ComponentVerifyTicket"]
		err := c.wechatServ.UpdateComponentTicket(conf)
		if err != nil {
			utils.LogError("修改ticket失败：", err)
		}
		utils.RefreshComponentTicket(msg["ComponentVerifyTicket"])
		break
	case INFOTYPE_AUTHORIZED:
		// 授权成功
		var cert = map[string]string{
			"appid":              msg["AuthorizerAppid"],
			"authorization_code": msg["AuthorizationCode"],
		}
		beego.Error("授权成功参数：", cert)
		wxclient := utils.WechatInit(cert, c.wechatServ.UpdateToken)
		if wxclient == nil {
			utils.LogError("获取wxclient失败")
			c.ResponseRaw(nilData)
		}
		beego.Error(wxclient)
		utils.AppendWxClient(wxclient)

		// 获取微信信息
		info, err := wxclient.GetWechatInfo()
		if err != nil {
			utils.LogError("获取微信信息失败")
			c.ResponseRaw(nilData)
		}
		beego.Error(info)
		authorizerInfo := (info["authorizer_info"]).(map[string]interface{})
		beego.Error(info["authorizer_info"])
		// 插入数据库
		mjson, _ := json.Marshal(msg)
		HeadImg := ""
		if authorizerInfo["head_img"] != nil {
			HeadImg = authorizerInfo["head_img"].(string)
		}
		var conf = model.SysWechatConf{
			Appid:             msg["AuthorizerAppid"],
			AuthorizationCode: msg["AuthorizationCode"],
			AuthorizationInfo: string(mjson),
			WxNikeName:        authorizerInfo["nick_name"].(string),
			HeadImg:           HeadImg,
			UserName:          authorizerInfo["user_name"].(string),
			PrincipalName:     authorizerInfo["principal_name"].(string),
			QrcodeUrl:         authorizerInfo["qrcode_url"].(string),
		}

		err = c.wechatServ.SaveWechatConf(conf)
		if err != nil {
			utils.LogError("保存微信授权信息失败：", err)
			c.ResponseRaw(nilData)
		}
		break
	case INFOTYPE_UPDATEAUTHORIZED:
		// 授权更新
		break
	case INFOTYPE_UNAUTHORIZED:
		// 取消授权
		// 删除wechatConf信息，同时将org解绑
		appid := msg["AuthorizerAppid"]
		c.wechatServ.UnAuthorized(appid)
		// 删除第三方中的微信信息
		utils.Component.DelWxClient(appid)
		break
	}
	c.ResponseRaw(nilData)
}

// WechatInfo 微信接入
func (c *WechatController) WechatInfo() {
	method := c.Ctx.Input.Method()
	if method == http.MethodGet {
		echostr := c.GetString("echostr")
		c.ResponseRaw([]byte(echostr))
	} else {
		c.WxReceive()
	}
}

// WxReceive 微信消息接收
func (c *WechatController) WxReceive() {
	utils.LogInfo("接收微信事件, URI: ", c.Ctx.Input.URI())

	// 初始化微信客户端
	appid := c.GetString(":appid")
	wechat, err := utils.Component.GetWxClient(appid)
	if err != nil {
		utils.LogError("获取微信失败: " + err.Error())
		c.ResponseRaw(nilData)
	}

	conf, err := c.wechatServ.GetComponentInfo()
	if err != nil || conf == nil || conf.Appid == "" {
		utils.LogError("读取微信配置文件失败", err)
		c.ResponseRaw(nilData)
	}

	// 校验接收数据
	receiveData, err := c.validReceiveData(c.Ctx.Input.RequestBody, conf.Aeskey, wechat)
	if err != nil {
		c.ResponseRaw(nilData)
	}

	// 获取返回内容
	replyMessage, err := c.getReplayMessage(receiveData, wechat)
	if err != nil || replyMessage == nil || len(replyMessage) == 0 {
		c.ResponseRaw(nilData)
	}

	utils.LogInfo("反馈微信内容: ", string(replyMessage))

	// 加密内容
	sendData, err := c.encryptMessage(replyMessage, conf.Aeskey, conf.OriginToken, wechat)
	if err != nil {
		c.ResponseRaw(nilData)
	}

	sendMessage := string(sendData)

	// 去掉多余的回车
	sendMessage = strings.Replace(sendMessage, "\n\n", "\n", -1)
	sendMessage = strings.Trim(sendMessage, "\n")
	utils.LogInfo("反馈加密内容: ", sendMessage)

	c.Ctx.Output.Header("Content-Type", "text/xml; charset=utf-8")
	c.ResponseRaw([]byte(sendMessage))
}

// validReceiveData 校验接收信息
// 没有通过 signature 校验，默认是正确的
func (c *WechatController) validReceiveData(receiveData []byte, aesKey string, wechat *component.WxClient) (data map[string]string, err error) {
	receiveMessage := string(receiveData)
	utils.LogInfo("事件内容: ", receiveMessage)

	var msg map[string]string
	msg, err = wechat.TransformMessage(receiveMessage)
	if err != nil {
		utils.LogError("读取微信服务发送内容失败: " + err.Error())
		return
	}

	var key []byte
	var encryptData []byte
	var decryptData []byte

	key, err = base64.StdEncoding.DecodeString(aesKey + "=")
	if err != nil {
		utils.LogError("Base64 decode aes-key 失败：", err)
		return
	}

	encryptData, err = base64.StdEncoding.DecodeString(msg["Encrypt"])
	if err != nil {
		utils.LogError("密文base64解析", err)
		return
	}

	decryptData, err = utils.AesDecrypt(encryptData, key)
	if err != nil {
		utils.LogError("解密失败：", err)
		return
	}

	utils.LogInfo("内容解密: ", string(decryptData))

	// 解析xml
	xp := &core.XMLParse{}
	data, err = xp.Parse(string(decryptData))
	if err != nil {
		utils.LogError("xml解析失败：", err)
		return
	}

	return
}

// encryptReceiveMessage 加密需要发送的信息
func (c *WechatController) encryptMessage(message []byte, aesKey, token string, wechat *component.WxClient) ([]byte, error) {
	cipher, err := core.NewCipher(token, aesKey, wechat.GetAppID())
	// cipher, err := core.NewCipher("pamtest", "abcdefghijklmnopqrstuvwxyz0123456789ABCDEFG", "wxb11529c136998cb6")
	if err != nil {
		utils.LogError("NewCipher 失败: " + err.Error())
		return nil, err
	}

	dt := bytes.NewBuffer([]byte{})

	nonce := c.GetString("nonce")
	timestamp := c.GetString("timestamp")

	cipher.Encrypt(dt, message, nonce, timestamp)

	return dt.Bytes(), nil
}

func (c *WechatController) getReplayMessage(receviceData map[string]string, wechat *component.WxClient) (message []byte, err error) {
	var replay *model.TaAutoReply

	msgType := receviceData["MsgType"]
	from := receviceData["ToUserName"]
	openID := receviceData["FromUserName"]
	appID := wechat.GetAppID()

	// 暂时支持 文本以及订阅事件消息
	switch msgType {
	case "text":
		if replay, err = c.serv.GetAutoReplayByAppID(appID, receviceData["Content"]); err != nil {
			utils.LogError("获取微信自动回复信息失败: " + err.Error())
			return
		}

		break
	case "event":
		switch receviceData["Event"] {
		case "subscribe":
			if replay, err = c.serv.GetSubscribeByAppID(appID); err != nil {
				utils.LogError("获取微信自动回复信息失败: " + err.Error())
				return
			}

			break
		case "CLICK":
			target := receviceData["EventKey"]
			if target != "" {
				var replyText string
				replyText, err = c.wechatServ.GetMenuReplyText(target)
				if err != nil {
					utils.LogError("获取菜单回复信息失败: " + err.Error())
					return
				}

				replay = &model.TaAutoReply{
					MessageType:      models.MESSAGE_TYPE_PARAGRAPH,
					MessageParagraph: replyText,
					IsUse:            models.AUTOREPLY_IS_USE_ON,
				}
			}
			break
		}

		break
	}

	if replay == nil {
		return
	}

	switch replay.MessageType {
	case models.MESSAGE_TYPE_PARAGRAPH:
		return wechat.ResponseMessageText(from, openID, replay.MessageParagraph)
	case models.MESSAGE_TYPE_IMG:
		return wechat.ResponseMessageImage(from, openID, replay.MessageImg)
	}
	return
}

// GetPreAuthCode 获取预授权码
func (c *WechatController) GetPreAuthCode() {
	code, err := utils.Component.GetPreAuthCode()
	if err != nil {
		utils.LogError("获取预授权码错误: " + err.Error())
		c.ResponseError(err)
	}
	appid := utils.Component.GetCertificate()["appid"]
	c.ResponseJSON(map[string]string{
		"appid": appid,
		"code":  code,
	})
}

// GetWechatMenuByAppID 根据appid获取微信详情
func (c *WechatController) GetWechatInfoByAppID() {
	appid := c.GetString(":appid")
	wxclient, err := utils.Component.GetWxClient(appid)
	if err != nil {
		utils.LogError("获取微信信息失败: " + err.Error())
		c.ResponseError(err)
	}
	info, err := wxclient.GetWechatInfo()
	if err != nil {
		utils.LogError("获取微信详情失败: " + err.Error())
		c.ResponseError(err)
	}
	c.ResponseJSON(info)
}

// GetWechatMenuByAppID 获取菜单
func (c *WechatController) GetMaterialByAppID() {
	appid := c.GetString(":appid")
	wxclient, err := utils.Component.GetWxClient(appid)
	if err != nil {
		utils.LogError("获取微信信息失败: " + err.Error())
		c.ResponseError(err)
	}
	var data = map[string]string{}
	data["offset"] = "0"
	data["type"] = "image"
	data["count"] = "10"

	menus, err := wxclient.GetMaterialList(data)
	if err != nil {
		utils.LogError("获取微信详情失败: " + err.Error())
		c.ResponseError(err)
	}
	c.ResponseJSON(menus)
}

// GetWechatByCode 根据code获取微信信息
func (c *WechatController) GetWechatByCode() {
	code := c.GetString(":code")
	beego.Error(code)
	conf, err := c.wechatServ.GetWechatByCode(code)
	if err != nil {
		utils.LogError("获取微信详情失败: " + err.Error())
		c.ResponseError(err)
	}
	beego.Error(conf)
	if conf == nil || conf.ConfId == "" || conf.Status == models.STATUS_NORMAL {
		c.ResponseJSON("")
	}

	c.ResponseJSON(map[string]string{
		"ConfId":   conf.ConfId,
		"NickName": conf.WxNikeName,
		"HeadImg":  conf.HeadImg,
	})
}
